# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sushy

from sushycli import base_system


class SystemPowerShow(base_system.BaseLister):
    """Show machine power state"""

    def take_action(self, args):
        """Power state management command action.

        :param args: a namespace of command-line option-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(SystemPowerShow, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        return ['Power state'], [[sys_inst.power_state]]


class SystemPowerSet(base_system.BaseCommand):
    """Change machine power state"""

    def get_parser(self, prog_name):
        """Power state management command parser.

        :param prog_name: name of the cliff command being executed
        :returns: an `argparse.ArgumentParser` instance
        """
        parser = super(SystemPowerSet, self).get_parser(prog_name)

        parser.add_argument(
            'state',
            metavar='on|off',
            type=lambda x: x.lower(),
            choices=['on', 'off'],
            help='Set machine power state')

        return parser

    def take_action(self, args):
        """Power state management command action.

        :param args: a namespace of command-line option-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(SystemPowerSet, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        sys_inst.reset_system(
            sushy.RESET_TYPE_ON
            if args.state == 'on' else sushy.RESET_TYPE_FORCE_OFF)

        return 0



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushycli import base_manager
from sushycli import utils


class ManagerInventoryShow(base_manager.BaseLister):
    """Show Manager inventory information"""

    def take_action(self, args):
        """Show manager inventory information command action.

        :param args: a namespace of command-line option-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(ManagerInventoryShow, self).take_action(args)

        mgr_inst = root.get_manager(args.manager_id)

        columns = [
            'Identity', 'Name', 'Description', 'OEM Vendors'
        ]

        return columns, [[utils.get_resource_column(mgr_inst, column)
                          for column in columns]]



# -*- coding: utf-8 -*-

# Copyright 2020 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from sushycli.base import BaseParserMixIn
from sushycli.tests.unit import base


class BaseParserMixInTestCase(base.TestCase):

    def setUp(self):
        super(BaseParserMixInTestCase, self).setUp()

        self.obj = BaseParserMixIn()

    def test__to_json_from_none(self):
        output = self.obj._to_json(None)

        expected = '{}'

        self.assertEqual(expected, output)

    def test__to_json_from_dict(self):
        output = self.obj._to_json({'test': 1})

        expected = '{\n  "test": 1\n}'

        self.assertEqual(expected, output)

    def test__to_json_from_json(self):
        output = self.obj._to_json('{"test": 1}')

        expected = '{\n  "test": 1\n}'

        self.assertEqual(expected, output)

    def test__to_json_malformed(self):
        self.obj.app = mock.MagicMock()

        output = self.obj._to_json('not json')

        expected = '{}'

        self.assertEqual(expected, output)
        self.assertEqual(1, self.obj.app.LOG.error.call_count)



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse

from sushycli import base_system


class SystemBiosShow(base_system.BaseLister):
    """Show machine BIOS attributes"""
    def take_action(self, args):
        """System BIOS show command action

        :param args: a namespace of command-line attribute-value
            pairs that come from the user
        :returns: columns, values of data to be listed.
        """

        root = super(SystemBiosShow, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        return (['BIOS attribute', 'BIOS value'],
                [[attribute[0], attribute[1]]
                for attribute in sys_inst.bios.attributes.items()])


class SystemBiosReset(base_system.BaseCommand):
    """Reset machine BIOS attributes to default"""

    def take_action(self, args):
        """System BIOS reset command action

        :param args: a namespace of command-line attribute-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(SystemBiosReset, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        sys_inst.bios.reset_bios()

        return 0


class SystemBiosSet(base_system.BaseCommand):
    """Update the system BIOS attribute(s)

    This command allows updating BIOS attributes, On the command line,
    a declaration should typically be a number of key-value pairs.

    :Example:  foo=hello or foo="hello"

    .. note:: Attributes update is not immediate but might require
        system restart.
    """

    def get_parser(self, prog_name):
        """Set system BIOS command parser

        :param prog_name: name of the cliff command being executed
        :returns: an `argparse.ArgumentParser` instance
        """
        parser = super(SystemBiosSet, self).get_parser(prog_name)

        parser.add_argument(
            'set',
            nargs='+',
            metavar="KEY=VALUE",
            action=ParseDict,
            help='Set BIOS attributes, Note that you should not '
                 'put spaces before or after the = sign.\n'
                 'If a value contains spaces, you should define '
                 'it with double quotes, for example:\n '
                 'foo="this is a sentence".')

        return parser

    def take_action(self, args):
        """Set system BIOS command action

        :param args: a namespace of command-line attribute-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(SystemBiosSet, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        sys_inst.bios.set_attributes(args.set)
        return 0


class ParseDict(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        d = {}

        if values:
            for item in values:
                split_items = item.split("=", 1)
                # we remove blanks around keys
                key = split_items[0].strip()
                value = split_items[1]

                d[key] = value

        setattr(namespace, self.dest, d)



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushycli import base_vmedia


class ManagerVmediaList(base_vmedia.BaseLister):
    """Display a list of available virtual media devices identities."""

    def take_action(self, args):
        """List available virtual media devices command action"""

        root = super(ManagerVmediaList, self).take_action(args)

        mgr_inst = root.get_manager(args.manager_id)

        return (['Virtual Media ID'], [[member]
                for member in mgr_inst.virtual_media.members_identities])


class ManagerVmediaInsert(base_vmedia.BaseCommand):
    """Insert a remote media into the virtual media device.

    This command allows to insert a HTTP-based image into the virtual device.

    Note:
        All systems being managed by this manager and booting from their
        corresponding removable media device (e.g. cdrom or fd) will boot
        the image inserted into manager’s virtual media device.
    """

    def get_parser(self, prog_name):
        """Insert virtual media command parser"""

        parser = super(ManagerVmediaInsert, self).get_parser(prog_name)

        parser.add_argument(
            '--image',
            required=True,
            help='The image path for virtual media access. '
                 'The value is a HTTP based URL.')
        return parser

    def take_action(self, args):
        """Insert virtual media command action"""

        root = super(ManagerVmediaInsert, self).take_action(args)

        mgr_inst = root.get_manager(args.manager_id)

        vm_inst = mgr_inst.virtual_media.get_member(args.device_id)

        vm_inst.insert_media(args.image)

        return 0


class ManagerVmediaEject(base_vmedia.BaseCommand):
    """Detach one or all the virtual media device(s).

    Note:
        All removable media device of all systems being
        managed by this manager will be ejected.
    """

    MULTIPLE_SYSTEMS_WARNING = """"
WARNING! Multiple managed systems may be affected! If you are sure that
you want to continue this operation, please use --force argument!
"""

    def get_parser(self, prog_name):
        """Eject virtual media command parser"""

        parser = super(ManagerVmediaEject, self).get_parser(prog_name)

        parser.add_argument(
            '--force',
            nargs='?',
            const='Yes',
            help='Virtual media force eject when multiple '
                 'systems has been managed by the manager')
        return parser

    def take_action(self, args):
        """Eject virtual media command action"""

        root = super(ManagerVmediaEject, self).take_action(args)

        mgr_inst = root.get_manager(args.manager_id)

        virtmedia_col = mgr_inst.virtual_media

        vm_inst = virtmedia_col.get_member(args.device_id)

        if len(mgr_inst.systems) > 1 and not args.force:
            print(self.MULTIPLE_SYSTEMS_WARNING)
            return 0

        vm_inst.eject_media()

        return 0



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sushy

from sushycli import base_system


class SystemBootShow(base_system.BaseLister):
    """Show system boot information"""

    def take_action(self, args):
        """Show system boot information command action"""

        root = super(SystemBootShow, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        colunms = [
            'Boot mode',
            'Available boot modes',
            'Boot device',
            'Available boot devices'
        ]

        boot_modes = sorted(sys_inst.boot.allowed_values)
        boot_devices = sorted(
            sys_inst.get_allowed_system_boot_source_values())

        values = [[sys_inst.boot.mode, ', '.join(boot_modes),
                   sys_inst.boot.target, ', '.join(boot_devices)]]

        return colunms, values


class SystemBootSet(base_system.BaseCommand):
    """Set the system boot mode/device

    Change system boot mode and device specifying the frequency;
    either it is disabled, applied once or persistent for future reboots
    """

    def get_parser(self, prog_name):
        """Set boot device command parser"""

        parser = super(SystemBootSet, self).get_parser(prog_name)

        parser.add_argument(
            '--target',
            choices=['none', 'pxe', 'floppy', 'cd', 'usb', 'hdd'],
            type=lambda x: x.lower(),
            required=True,
            help='the target boot source')
        parser.add_argument(
            '--enabled',
            type=lambda x: x.lower(),
            choices=['disabled', 'once', 'continuous'],
            help='Next reboot only or persistent to all future reboots, '
                 'default is set to boot once')
        parser.add_argument(
            '--mode',
            choices=['bios', 'uefi'],
            type=lambda x: x.lower(),
            help='the machine boot mode')

        return parser

    def take_action(self, args):
        """Set boot device command action"""

        root = super(SystemBootSet, self).take_action(args)

        sys_inst = root.get_system(args.system_id)

        sys_inst.set_system_boot_source(
            args.target,
            enabled=args.enabled,
            mode=sushy.BOOT_SOURCE_MODE_UEFI
            if args.mode == 'uefi' else sushy.BOOT_SOURCE_MODE_BIOS)
        return 0



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import json
import sys
from urllib.parse import urlsplit
from urllib.parse import urlunsplit

from cliff import command
from cliff import lister
import sushy
from sushy import connector
import urllib3


class BaseParserMixIn(object):
    """Common bits and pieces of all `sushycli` commands.

    Does not implement any CLI command by its own.
    """

    def add_parser_options(self, parser):

        parser.add_argument(
            '--username',
            help='Redfish BMC username')

        parser.add_argument(
            '--password',
            help='Redfish BMC user password')

        parser.add_argument(
            '--service-endpoint',
            required=True,
            help='Redfish BMC service endpoint URL (e.g. '
                 'http://localhost:8000). If location part of the URL is not '
                 'specified, it defaults to `/redfish/v1`.')

        parser.add_argument(
            '--insecure', action='store_true',
            help='Do not verify server TLS certificate')

        parser.add_argument(
            '--tls-certificates', metavar='<FILE|DIR>',
            help='Path to a CA bundle or a directory containing trusted '
                 'TLS certificates.')

        parser.add_argument(
            '--show-traffic',
            action='store_true',
            help='Show Redfish HTTP message exchange.')

        return parser

    def _to_json(self, data):
        """Turn input object into JSON when possible.

        :param data: object to jsonize
        :type data: dict or str or None
        """
        if not data:
            data = {}

        elif isinstance(data, (str, bytes)):
            try:
                data = json.loads(data)

            except json.JSONDecodeError as exc:
                self.app.LOG.error(
                    'Malformed JSON document %(doc)s: %(error)s',
                    {'doc': data, 'error': exc})
                data = {}

        else:
            data = dict(data)

        return json.dumps(data, indent=2)

    def observer(self, response):
        """Pretty print HTTP request and response details.

        :param response: HTTP response object
        :type response: requests.Response
        """
        sys.stdout.write(
            '%s %s\n%s\n%s\n' % (
                response.request.method, response.request.url,
                self._to_json(response.request.headers),
                self._to_json(response.request.body)))

        sys.stdout.write(
            '%s %s\n%s\n%s\n' % (
                response.reason, response.status_code,
                self._to_json(response.headers),
                self._to_json(response.text)))

        sys.stdout.flush()

    def take_action(self, args):
        """Common base for all command actions

        :param args: a namespace of command-line option-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        url = urlsplit(args.service_endpoint, scheme='http')

        address = []
        path = []

        for idx, component in enumerate(url):
            if idx < 2:
                address.append(component)
                path.append('')
            else:
                address.append('')
                path.append(component)

        base_url = urlunsplit(address)

        kwargs = {
            'base_url': base_url,
            'username': args.username,
            'password': args.password,
        }

        verify = args.tls_certificates or not args.insecure
        if verify is not True:
            kwargs.update(verify=verify)

        path = urlunsplit(path)
        if path:
            kwargs.update(root_prefix=path)

        if args.insecure:
            urllib3.disable_warnings(
                urllib3.exceptions.InsecureRequestWarning)

        conn = connector.Connector(
            base_url, verify=verify,
            response_callback=args.show_traffic and self.observer)

        kwargs.update(connector=conn)

        return sushy.Sushy(**kwargs)


class BaseCommand(BaseParserMixIn, command.Command):
    """Common base for all sushycli status commands"""

    def get_parser(self, prog_name):
        """Common base for all status command parsers.

        :param prog_name: name of the cliff command being executed
        :returns: an `argparse.ArgumentParser` instance
        """

        parser = super(BaseCommand, self).get_parser(prog_name)

        # Get correctly formatted help description
        parser.formatter_class = argparse.RawDescriptionHelpFormatter

        return self.add_parser_options(parser)


class BaseLister(BaseParserMixIn, lister.Lister):
    """Common base for all sushycli listing commands"""

    def get_parser(self, prog_name):
        """Common base for all listing command parsers.

        :param prog_name: name of the cliff command being executed
        :returns: an `argparse.ArgumentParser` instance
        """
        parser = super(BaseLister, self).get_parser(prog_name)

        return self.add_parser_options(parser)



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushycli import base


class ChassisList(base.BaseLister):
    """List available chassis"""

    def get_parser(self, prog_name):
        """List chassis command parser.

        :param prog_name: name of the cliff command being executed
        :returns: an `argparse.ArgumentParser` instance
        """
        parser = super(ChassisList, self).get_parser(prog_name)

        return parser

    def take_action(self, args):
        """List chassis command action.

        :param args: a namespace of command-line option-value pairs that
            come from the user
        :returns: CLI process exit code
        """
        root = super(ChassisList, self).take_action(args)

        chassis = root.get_chassis_collection()

        rows = []

        for chassis in chassis.get_members():
            chassis_inst = root.get_chassis(chassis.path)

            rows.append(
                [chassis_inst.name, chassis_inst.uuid, chassis.path]
            )

        columns = [
            'Name', 'Identity', 'Chassis ID'
        ]

        return columns, rows



# -*- coding: utf-8 -*-

# Copyright 2010-2020 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushycli import base


class BaseParserMixIn(base.BaseParserMixIn):
    """Common bits and pieces of all `sushycli system` commands.

    Does not implement any CLI command by its own.
    """
    SYSTEM_ID_HELP = (
        'The canonical path to the ComputerSystem resource to interact with. '
        'It should include the root service, version and the unique resource '
        'path to a ComputerSystem. For example: /redfish/v1/Systems/1'
    )

    def add_parser_options(self, parser):

        parser = super(BaseParserMixIn, self).add_parser_options(parser)

        parser.add_argument(
            '--system-id',
            help=self.SYSTEM_ID_HELP)

        return parser


class BaseCommand(BaseParserMixIn, base.BaseCommand):
    """Common base for all sushycli system status commands"""


class BaseLister(BaseParserMixIn, base.BaseLister):
    """Common base for all sushycli system listing commands"""



# -*- coding: utf-8 -*-

# Copyright 2020 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import sushy

from sushycli.cmd.sushycli import main
from sushycli.tests.unit import base


@mock.patch.object(sushy, 'Sushy', autospec=True)
@mock.patch.object(sushy.connector, 'Connector', autospec=True)
class SuchyCliTestCase(base.TestCase):

    @mock.patch('sys.stdout.write', autospec=True)
    def test_show_traffic(self, mock_write, mock_connector, mock_sushy):

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me', '--show-traffic'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=mock.ANY, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

    @mock.patch('sys.stdout.write', autospec=True)
    def test_insecure(self, mock_write, mock_connector, mock_sushy):

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me', '--insecure'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=False)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly', verify=False)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_tls_certificate(self, mock_write, mock_connector, mock_sushy):

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--tls-certificate', '/dev/null'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify='/dev/null')

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly', verify='/dev/null')

    @mock.patch('sys.stdout.write', autospec=True)
    def test_service_endpoint_default(
            self, mock_write, mock_connector, mock_sushy):

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

    @mock.patch('sys.stdout.write', autospec=True)
    def test_service_endpoint_mounted(
            self, mock_write, mock_connector, mock_sushy):

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me:1234/out'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me:1234', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me:1234', connector=mock_connection,
            password='fish', username='jelly',
            root_prefix='/out')

    @mock.patch('sys.stdout.write', autospec=True)
    def test_version(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_root.redfish_version = '1.2.3'

        main(['version', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+---------+\n'
                      '| Version |'
                      '\n+---------+\n'
                      '| 1.2.3   |\n'
                      '+---------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_manager_vmedia_list(
            self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_manager = mock_root.get_manager.return_value

        mock_manager.virtual_media.\
            members_identities = (
                '/redfish/v1/Managers/58893887-8974-2487-2389-841168418919/'
                'VirtualMedia/Cd',
                '/redfish/v1/Managers/58893887-8974-2487-2389-841168418919/'
                'VirtualMedia/Floppy'
            )

        main(['manager', 'vmedia', 'list',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--manager-id', '/redfish/v1/Managers/BMC'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+-----------------------------------------'
                      '--------------------------------------+\n'
                      '| Virtual Media ID                       '
                      '                                       |\n'
                      '+-----------------------------------------'
                      '--------------------------------------+\n'
                      '| /redfish/v1/Managers/58893887-8974-2487-'
                      '2389-841168418919/VirtualMedia/Cd     |\n'
                      '| /redfish/v1/Managers/58893887-8974-2487-'
                      '2389-841168418919/VirtualMedia/Floppy |\n'
                      '+----------------------------------------'
                      '---------------------------------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    def test_manager_vmedia_insert(self, mock_connector, mock_sushy):

        main(['manager', 'vmedia', 'insert',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--manager-id', '/redfish/v1/Managers/BMC',
              '--device-id', '/redfish/v1/Managers/BMC/VirtualMedia/Cd',
              '--image', 'http://fish.me/fishiso.iso'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_manager.assert_called_once_with(
            '/redfish/v1/Managers/BMC')

        mock_manager = mock_root.get_manager.return_value

        mock_vmedia = mock_manager.virtual_media.\
            get_member('/redfish/v1/Managers/BMC/VirtualMedia/Cd')
        mock_vmedia.insert_media.\
            assert_called_once_with('http://fish.me/fishiso.iso')

    def test_manager_vmedia_eject(self, mock_connector, mock_sushy):

        main(['manager', 'vmedia', 'eject',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--manager-id', '/redfish/v1/Managers/BMC',
              '--device-id', '/redfish/v1/Managers/BMC/VirtualMedia/Cd'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_manager.assert_called_once_with(
            '/redfish/v1/Managers/BMC')

        mock_manager = mock_root.get_manager.return_value

        mock_vmedia = mock_manager.virtual_media\
            .get_member('/redfish/v1/Managers/BMC/VirtualMedia/Cd')
        mock_vmedia.eject_media.assert_called_once()

    @mock.patch('sys.stdout.write', autospec=True)
    def test_system_boot_show(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value

        mock_system.boot.mode = 'Disabled'
        mock_system.boot.allowed_values = ['Pxe', 'Cd', 'Hdd']
        mock_gasbsv = mock_system.get_allowed_system_boot_source_values
        mock_gasbsv.return_value = {'hdd', 'pxe', 'cd'}
        mock_system.boot.target = 'Hdd'

        main(['system', 'boot', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+-----------+----------------------+-------------+----'
                      '--------------------+\n| Boot mode | Available boot mo'
                      'des | Boot device | Available boot devices |\n+-------'
                      '----+----------------------+-------------+------------'
                      '------------+\n| Disabled  | Cd, Hdd, Pxe         | Hd'
                      'd         | cd, hdd, pxe           |\n+-----------+---'
                      '-------------------+-------------+--------------------'
                      '----+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_system_bios_show(self, mock_write, mock_connector, mock_sushy):
        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value
        mock_system.bios.description = None
        mock_system.bios.attributes = {'BootMode': 'Uefi',
                                       'EmbeddedSata': 'Raid',
                                       'NicBoot1': 'NetworkBoot',
                                       'ProcTurboMode': 'Enabled'}

        main(['system', 'bios', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+----------------+-------------+\n'
                      '| BIOS attribute | BIOS value  |\n'
                      '+----------------+-------------+\n'
                      '| BootMode       | Uefi        |\n'
                      '| EmbeddedSata   | Raid        |\n'
                      '| NicBoot1       | NetworkBoot |\n'
                      '| ProcTurboMode  | Enabled     |\n'
                      '+----------------+-------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    def test_system_bios_reset(self, mock_connector, mock_sushy):

        main(['system', 'bios', 'reset',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_system.assert_called_once_with(
            '/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.bios.reset_bios.assert_called_once()

    def test_system_bios_set(self, mock_connector, mock_sushy):

        main(['system', 'bios', 'set',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1',
              'BootMode=Uefi', 'EmbeddedSata=Raid',
              'NicBoot1=NetworkBoot', 'ProcTurboMode=Enabled'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_system.assert_called_once_with(
            '/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.bios.set_attributes.assert_called_once_with(
            {
                "BootMode": "Uefi",
                "EmbeddedSata": "Raid",
                "NicBoot1": "NetworkBoot",
                "ProcTurboMode": "Enabled"
            }
        )

    def test_system_bios_set_with_keys_blanks(
            self, mock_connector, mock_sushy):

        main(['system', 'bios', 'set',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1',
              'BootMode  =Uefi', '  EmbeddedSata=Raid'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_system.assert_called_once_with(
            '/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.bios.set_attributes.assert_called_once_with(
            {
                "BootMode": "Uefi",
                "EmbeddedSata": "Raid",
            }
        )

    def test_system_boot_set(self, mock_connector, mock_sushy):

        main(['system', 'boot', 'set',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1',
              '--target', 'cd',
              '--enabled', 'once',
              '--mode', 'uefi'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root = mock_sushy.return_value

        mock_root.get_system.assert_called_once_with(
            '/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.set_system_boot_source.\
            assert_called_once_with(sushy.BOOT_SOURCE_TARGET_CD,
                                    enabled=sushy.BOOT_SOURCE_ENABLED_ONCE,
                                    mode=sushy.BOOT_SOURCE_MODE_UEFI)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_chassis_inventory_show(
            self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_one_chassis = mock_root.get_chassis.return_value

        mock_one_chassis.identity = 'A'
        mock_one_chassis.name = 'B'
        mock_one_chassis.description = 'C'
        mock_one_chassis.manufacturer = 'D'
        mock_one_chassis.part_number = 'E'
        mock_one_chassis.serial_number = 'F'
        mock_one_chassis.sku = 'G'
        mock_one_chassis.asset_tag = 'H'
        mock_one_chassis.oem_vendors = ['I', 'J']

        main(['chassis', 'inventory', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--chassis-id', '/redfish/v1/Chassis/1U'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+----------+------+-------------+--------------+-----'
                      '--------+---------------+-----+-----------+----------'
                      '---+\n| Identity | Name | Description | Manufacturer '
                      '| Part Number | Serial Number | SKU | Asset Tag | OEM'
                      ' Vendors |\n+----------+------+-------------+--------'
                      '------+-------------+---------------+-----+----------'
                      '-+-------------+\n| A        | B    | C           | D'
                      '            | E           | F             | G   | H  '
                      '       | I, J        |\n+----------+------+----------'
                      '---+--------------+-------------+---------------+----'
                      '-+-----------+-------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_manager_inventory_show(
            self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_manager = mock_root.get_manager.return_value

        mock_manager.identity = 'A'
        mock_manager.name = 'B'
        mock_manager.description = 'C'
        mock_manager.manufacturer = 'D'
        mock_manager.part_number = 'E'
        mock_manager.serial_number = 'F'
        mock_manager.sku = 'G'
        mock_manager.asset_tag = 'H'
        mock_manager.oem_vendors = ['I', 'J']

        main(['manager', 'inventory', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--manager-id', '/redfish/v1/Mnagers/BMC'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [

            mock.call('+----------+------+-------------+-------------+\n| Id'
                      'entity | Name | Description | OEM Vendors |\n+-------'
                      '---+------+-------------+-------------+\n| A        |'
                      ' B    | C           | I, J        |\n+----------+----'
                      '--+-------------+-------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_system_list(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock.MagicMock()

        mock_system.identity = 'A'
        mock_system.name = 'B'
        mock_system.uuid = '38947555-7742-3448-3784-823347823834'
        mock_system.path = '/redfish/v1/Systems/437XR1138R2'

        mock_systems = mock_root.get_system_collection.return_value
        mock_systems.get_members.return_value = [mock_system]
        mock_root.get_system.return_value = mock_system

        main(['system', 'list',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+------+--------------------------------------+------'
                      '---------------------------+\n| Name | Identity      '
                      '                       | System ID                   '
                      '    |\n+------+--------------------------------------'
                      '+---------------------------------+\n| B    | 3894755'
                      '5-7742-3448-3784-823347823834 | /redfish/v1/Systems/4'
                      '37XR1138R2 |\n+------+-------------------------------'
                      '-------+---------------------------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_manager_list(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_manager = mock.MagicMock()

        mock_manager.identity = 'A'
        mock_manager.name = 'B'
        mock_manager.uuid = '38947555-7742-3448-3784-823347823834'
        mock_manager.path = '/redfish/v1/Managers/BMC'

        mock_managers = mock_root.get_manager_collection.return_value
        mock_managers.get_members.return_value = [mock_manager]
        mock_root.get_manager.return_value = mock_manager

        main(['manager', 'list',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+------+--------------------------------------+------'
                      '--------------------+\n| Name | Identity             '
                      '                | Manager ID               |\n+------'
                      '+--------------------------------------+-------------'
                      '-------------+\n| B    | 38947555-7742-3448-3784-8233'
                      '47823834 | /redfish/v1/Managers/BMC |\n+------+------'
                      '--------------------------------+--------------------'
                      '------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_chassis_list(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_one_chassis = mock.MagicMock()

        mock_one_chassis.identity = 'A'
        mock_one_chassis.name = 'B'
        mock_one_chassis.uuid = '38947555-7742-3448-3784-823347823834'
        mock_one_chassis.path = '/redfish/v1/Chassis/1U'

        mock_chassis = mock_root.get_chassis_collection.return_value
        mock_chassis.get_members.return_value = [mock_one_chassis]
        mock_root.get_chassis.return_value = mock_one_chassis

        main(['chassis', 'list',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+------+--------------------------------------+------'
                      '------------------+\n| Name | Identity               '
                      '              | Chassis ID             |\n+------+---'
                      '-----------------------------------+-----------------'
                      '-------+\n| B    | 38947555-7742-3448-3784-8233478238'
                      '34 | /redfish/v1/Chassis/1U |\n+------+--------------'
                      '------------------------+------------------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_system_inventory_show(
            self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value

        mock_system.identity = 'A'
        mock_system.name = 'B'
        mock_system.description = 'C'
        mock_system.manufacturer = 'D'
        mock_system.part_number = 'E'
        mock_system.serial_number = 'F'
        mock_system.sku = 'G'
        mock_system.asset_tag = 'H'
        mock_system.oem_vendors = ['I', 'J']

        main(['system', 'inventory', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+----------+------+-------------+--------------+-----'
                      '--------+---------------+-----+-----------+----------'
                      '---+\n| Identity | Name | Description | Manufacturer '
                      '| Part Number | Serial Number | SKU | Asset Tag | OEM'
                      ' Vendors |\n+----------+------+-------------+--------'
                      '------+-------------+---------------+-----+----------'
                      '-+-------------+\n| A        | B    | C           | D'
                      '            | E           | F             | G   | H  '
                      '       | I, J        |\n+----------+------+----------'
                      '---+--------------+-------------+---------------+----'
                      '-+-----------+-------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    @mock.patch('sys.stdout.write', autospec=True)
    def test_system_power_show(self, mock_write, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value

        mock_system.power_state = 'on'

        main(['system', 'power', 'show',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        expected_calls = [
            mock.call('+-------------+\n'
                      '| Power state |\n'
                      '+-------------+\n'
                      '| on          |\n'
                      '+-------------+'),
            mock.call('\n')
        ]

        mock_write.assert_has_calls(expected_calls)

    def test_system_power_on(self, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value

        mock_system.power_state = 'off'

        main(['system', 'power',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1',
              'on'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root.get_system.assert_called_once_with('/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.reset_system.assert_called_once_with('on')

    def test_system_power_off(self, mock_connector, mock_sushy):

        mock_root = mock_sushy.return_value

        mock_system = mock_root.get_system.return_value

        mock_system.power_state = 'on'

        main(['system', 'power',
              '--username', 'jelly', '--password', 'fish',
              '--service-endpoint', 'http://fish.me',
              '--system-id', '/redfish/v1/Systems/1',
              'Off'])

        mock_connection = mock_connector.return_value

        mock_connector.assert_called_once_with(
            'http://fish.me', response_callback=False, verify=True)

        mock_sushy.assert_called_once_with(
            'http://fish.me', connector=mock_connection,
            password='fish', username='jelly')

        mock_root.get_system.assert_called_once_with('/redfish/v1/Systems/1')

        mock_system = mock_root.get_system.return_value

        mock_system.reset_system.assert_called_once_with('force off')
